class DataRequest:
    def __init__(self, topic: str, data: str):
        pass


class SubRequest:
    def __init__(self, topic: str):
        pass


class UnsubRequest:
    def __init__(self, topic: str):
        pass


class DataResponse:
    def __init__(self, topic: str, data: str):
        pass


class Empty:
    pass
